@@ -6,7 +6,7 @@ use Carp                   qw[];
 use Net::FastCGI::Constant qw[:all];
 
 BEGIN {
-    our $VERSION   = '0.10';
+    our $VERSION   = '0.11';
     our @EXPORT_OK = qw[ build_begin_request
                          build_begin_request_body
                          build_begin_request_record
@@ -31,6 +31,7 @@ BEGIN {
                          is_management_type
                          is_discrete_type
                          is_stream_type
+                         get_record_length
                          get_role_name
                          get_type_name
                          get_protocol_status_name ];
@@ -174,8 +175,10 @@ sub parse_record {
     (length $_[0] >= FCGI_HEADER_LEN + $content_length)
       || throw(ERRMSG_OCTETS, q/FCGI_Record/);
 
-    return parse_record_body($type, $request_id,
-      substr($_[0], FCGI_HEADER_LEN, $content_length));
+    return wantarray 
+      ? ($type, $request_id, substr($_[0], FCGI_HEADER_LEN, $content_length))
+      : parse_record_body($type, $request_id,
+          substr($_[0], FCGI_HEADER_LEN, $content_length));
 }
 
 sub parse_record_body {
@@ -183,12 +186,12 @@ sub parse_record_body {
     my ($type, $request_id) = @_;
 
     my $content_length = defined $_[2] ? length $_[2] : 0;
-    my %record = (type => $type, request_id => $request_id);
 
-    if ($content_length > FCGI_MAX_CONTENT_LEN) {
-        throw(ERRMSG_MALFORMED, $FCGI_RECORD_NAME[$type] || q/FCGI_Record/);
-    }
-    elsif ($type == FCGI_BEGIN_REQUEST) {
+    ($content_length <= FCGI_MAX_CONTENT_LEN)
+      || throw(ERRMSG_OCTETS_LE, q/content/, FCGI_MAX_CONTENT_LEN);
+
+    my %record = (type => $type, request_id => $request_id);
+    if ($type == FCGI_BEGIN_REQUEST) {
         ($request_id != FCGI_NULL_REQUEST_ID && $content_length == 8)
           || throw(ERRMSG_MALFORMED, q/FCGI_BeginRequestRecord/);
         @record{ qw(role flags) } = parse_begin_request_body($_[2]);
@@ -361,6 +364,14 @@ sub build_end_request {
     return $r;
 }
 
+sub get_record_length {
+    @_ == 1 || throw(q/Usage: get_record_length(octets)/);
+    (defined $_[0] && length $_[0] >= FCGI_HEADER_LEN)
+      || return 0;
+    return FCGI_HEADER_LEN + vec($_[0], 2, 16)  # contentLength
+                           + vec($_[0], 6,  8); # paddingLength
+}
+
 sub is_known_type {
     @_ == 1 || throw(q/Usage: is_known_type(type)/);
     my ($type) = @_;
